import { InjectionToken, Inject, Injectable, ChangeDetectorRef, Input, Component, ViewEncapsulation, ChangeDetectionStrategy, NgModule } from '@angular/core';
import { OverlayModule } from '@angular/cdk/overlay';
import { CommonModule } from '@angular/common';
import { NavigationEnd, ActivatedRoute, Router, RouterModule } from '@angular/router';
import { Subject, BehaviorSubject } from 'rxjs';
import { takeUntil, filter } from 'rxjs/operators';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { NtOverlayModule } from '@ng-tangram/components/overlay';

var NtNavUrlTreeStrategy;
(function (NtNavUrlTreeStrategy) {
    NtNavUrlTreeStrategy[NtNavUrlTreeStrategy["Integrity"] = 0] = "Integrity";
    NtNavUrlTreeStrategy[NtNavUrlTreeStrategy["OnlyParents"] = 1] = "OnlyParents";
})(NtNavUrlTreeStrategy || (NtNavUrlTreeStrategy = {}));
const NT_NAV_CONFIG = new InjectionToken('nt-nav-config');

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

function getCjsExportFromNamespace (n) {
	return n && n['default'] || n;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

function generateNavMenus(routes, rootDir) {
    return routes.filter(route => !route.redirectTo).map(route => {
        const menuRoles = (route.canActivate && route.canActivate.map((activate) => activate.ROLE_KEY)) || [];
        const menu = { path: route.path, data: route.data, roles: menuRoles.flat(Infinity) || [] };
        const subModule = commonjsRequire(`src/${rootDir}/${route.path}/${route.path}.module.ts`);
        if (subModule.routes) {
            menu.children = subModule.routes.filter(route => !route.redirectTo).map((route) => {
                const roles = (route.canActivate && route.canActivate.map((activate) => activate.ROLE_KEY)) || [];
                return { path: route.path, data: route.data, roles: roles.flat(Infinity) || [] };
            });
        }
        return menu;
    });
}
const RESOLVE_TITLE_PLACEHOLDER = /^resolve::/;
function resolveTitle(snapshot, title) {
    var _a;
    if (RESOLVE_TITLE_PLACEHOLDER.test(title)) {
        const paths = title.replace(RESOLVE_TITLE_PLACEHOLDER, '').split('.');
        const key = paths.shift();
        try {
            if (key && ((_a = snapshot.data) === null || _a === void 0 ? void 0 : _a.hasOwnProperty(key))) {
                const data = snapshot.data[key];
                let path = paths.shift();
                let result = path ? data[path] : null;
                while (paths.length > 0 && result instanceof Object) {
                    path = paths.shift();
                    result = path ? result[path] : null;
                }
                return result;
            }
            return key || '';
        }
        catch (_b) { }
    }
    return title;
}

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
let NtNavigationAdapter = class NtNavigationAdapter {
    constructor(route, router, config) {
        this.route = route;
        this.router = router;
        this.config = config;
        this.destroy = new Subject();
        this.activatedUrlTree = new BehaviorSubject([]);
        this.activeUrl = new BehaviorSubject({});
        const { routes, rootDir: rootDir } = config;
        this.menus = generateNavMenus(routes, rootDir);
        router.events
            .pipe(takeUntil(this.destroy), filter(event => event instanceof NavigationEnd))
            .subscribe(() => this.buildUrlTree(this.route));
    }
    ngOnDestroy() {
        this.destroy.next();
        this.destroy.complete();
    }
    getRouteMenus() {
        return this.menus;
    }
    buildUrlTree(route) {
        let url = [];
        const tree = [];
        while (route.firstChild) {
            route = route.firstChild;
            const snapshot = route.snapshot;
            if (snapshot.data && snapshot.data.title && snapshot.url.length > 0) {
                let data = snapshot.data, title;
                title = resolveTitle(snapshot, data.title);
                url = url.concat(snapshot.url.map(segment => segment.path));
                tree.push({ title, url, icon: data.icon });
            }
        }
        this.activeUrl.next(tree[tree.length - 1]);
        if (this.config.urlTreeStrategy === NtNavUrlTreeStrategy.OnlyParents) {
            tree.splice(tree.length - 1, 1);
        }
        this.activatedUrlTree.next(tree);
    }
};
NtNavigationAdapter.ctorParameters = () => [
    { type: ActivatedRoute },
    { type: Router },
    { type: undefined, decorators: [{ type: Inject, args: [NT_NAV_CONFIG,] }] }
];
NtNavigationAdapter = __decorate([
    Injectable(),
    __param(2, Inject(NT_NAV_CONFIG))
], NtNavigationAdapter);

var __decorate$1 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
let NtNavSidebarComponent = class NtNavSidebarComponent {
    constructor(changeDetectorRef, navAdapter) {
        this.changeDetectorRef = changeDetectorRef;
        this.navAdapter = navAdapter;
        this.destroy = new Subject();
        this._dropdown = false;
        this._baseUrl = navAdapter.config.baseUrl || './';
        this._routeMenus = this._filterRouteMenus();
        this.navAdapter.activatedUrlTree
            .pipe(takeUntil(this.destroy), filter(tree => tree.length > 0))
            .subscribe(tree => this._foldedActivatedMenu(tree));
    }
    get dropdown() { return this._dropdown; }
    set dropdown(value) {
        this._dropdown = coerceBooleanProperty(value);
    }
    set filter(value) {
        if (value) {
            this._filter = value;
            this._routeMenus = this._filterRouteMenus();
            this.changeDetectorRef.markForCheck();
        }
    }
    get filter() {
        return this._filter;
    }
    _foldedActivatedMenu(tree) {
        const moduleName = tree[0].url.join('/');
        const activated = this._routeMenus.find(menu => menu.path === moduleName);
        activated && (activated.folded = true);
        this.changeDetectorRef.markForCheck();
    }
    _filterRouteMenus() {
        if (typeof this.filter !== 'function') {
            return this.navAdapter.getRouteMenus();
        }
        const parents = this.navAdapter.getRouteMenus().filter(parent => {
            if (parent.children && parent.children.length > 0) {
                return true;
            }
            else {
                return this.filter(parent);
            }
        });
        const value = parents.map(parent => {
            const copy = Object.assign({}, parent);
            if (copy.children) {
                copy.children = copy.children.filter(child => this.filter(child));
            }
            return copy;
        });
        const result = [];
        value.forEach((menuItem) => {
            if (!menuItem.children || (menuItem.children && menuItem.children.length > 0)) {
                result.push(Object.assign({}, menuItem));
            }
        });
        return result;
    }
};
NtNavSidebarComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: NtNavigationAdapter }
];
__decorate$1([
    Input()
], NtNavSidebarComponent.prototype, "dropdown", null);
__decorate$1([
    Input()
], NtNavSidebarComponent.prototype, "filter", null);
NtNavSidebarComponent = __decorate$1([
    Component({
        selector: 'nt-nav-sidebar',
        template: "<ul class=\"nt-nav-sidebar-root-menu\">\n  <li *ngFor=\"let parent of _routeMenus\"\n    routerLinkActive=\"active\"\n    [class.folded]=\"parent.folded\"\n    [class.has-children]=\"parent.children\">\n\n    <ng-container *ngIf=\"!dropdown else dropdownMode\">\n      <a *ngIf=\"!parent.children else hasChildrenMenus\" [routerLink]=\"[_baseUrl, parent.path]\">\n        <i [class]=\"parent.data['icon']\"></i>\n        <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n      </a>\n      <ng-template #hasChildrenMenus>\n        <a (click)=\"parent.folded = !parent.folded\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n        </a>\n        <ul class=\"nt-nav-sidebar-nested-menu\">\n          <li *ngFor=\"let child of parent.children\" routerLinkActive=\"active\">\n            <a [routerLink]=\"[_baseUrl, parent.path, child.path]\">\n              <span class=\"nt-nav-sidebar-title\">{{ child.data['title'] }}</span>\n            </a>\n          </li>\n        </ul>\n      </ng-template>\n    </ng-container>\n\n    <ng-template #dropdownMode>\n      <ng-container *ngIf=\"!parent.children else hasChildrenMenus\">\n        <a #origin=\"cdkOverlayOrigin\"\n        cdkOverlayOrigin\n        (mouseenter)=\"overlay.markOpen()\"\n        (mouseleave)=\"overlay.markClose()\"\n        [routerLink]=\"[_baseUrl, parent.path]\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n\n          <nt-overlay\n            #overlay\n            overlayClass=\"nt-nav-dropdown-overlay\"\n            [origin]=\"origin\"\n            position=\"rightTop\"\n            [arrow]=\"true\"\n            (overlayEnter)=\"overlay.markOpen()\"\n            (overlayLeave)=\"overlay.markClose()\">\n            <div class=\"nt-nav-dropdown-pane\">\n              <div class=\"nt-nav-dropdown-title\">{{ parent.data['title'] }}</div>\n            </div>\n          </nt-overlay>\n        </a>\n      </ng-container>\n      <ng-template #hasChildrenMenus>\n        <a #origin=\"cdkOverlayOrigin\"\n          cdkOverlayOrigin\n          (mouseenter)=\"overlay.markOpen()\"\n          (mouseleave)=\"overlay.markClose()\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n          <nt-overlay\n            #overlay\n            overlayClass=\"nt-nav-dropdown-overlay\"\n            [origin]=\"origin\"\n            position=\"rightTop\"\n            [arrow]=\"true\"\n            (overlayEnter)=\"overlay.markOpen()\"\n            (overlayLeave)=\"overlay.markClose()\">\n            <div class=\"nt-nav-dropdown-pane\">\n              <div class=\"nt-nav-dropdown-title\">{{ parent.data['title'] }}</div>\n              <ul class=\"nt-nav-dropdown-menu\">\n                <li *ngFor=\"let child of parent.children\" routerLinkActive=\"active\">\n                  <a [routerLink]=\"[_baseUrl, parent.path, child.path]\">\n                    <span>{{ child.data['title'] }}</span>\n                  </a>\n                </li>\n              </ul>\n            </div>\n          </nt-overlay>\n        </a>\n      </ng-template>\n    </ng-template>\n  </li>\n</ul>\n\n",
        encapsulation: ViewEncapsulation.None,
        changeDetection: ChangeDetectionStrategy.OnPush,
        host: {
            'class': 'nt-nav-sidebar',
            '[class.dropdown-mode]': 'dropdown'
        }
    })
], NtNavSidebarComponent);

var __decorate$2 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
let NtNavTopbarComponent = class NtNavTopbarComponent {
    constructor(navAdapter) {
    }
};
NtNavTopbarComponent.ctorParameters = () => [
    { type: NtNavigationAdapter }
];
NtNavTopbarComponent = __decorate$2([
    Component({
        selector: 'nt-nav-topbar',
        template: "",
        encapsulation: ViewEncapsulation.None,
        host: {
            'class': 'nt-nav-topbar'
        }
    })
], NtNavTopbarComponent);

var __decorate$3 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtNavModule_1;
let NtNavModule = NtNavModule_1 = class NtNavModule {
    static forRoot(config) {
        return {
            ngModule: NtNavModule_1,
            providers: [
                { provide: NT_NAV_CONFIG, useValue: config }
            ]
        };
    }
};
NtNavModule = NtNavModule_1 = __decorate$3([
    NgModule({
        imports: [
            CommonModule,
            RouterModule,
            OverlayModule,
            NtOverlayModule,
        ],
        exports: [
            NtNavSidebarComponent,
            NtNavTopbarComponent,
        ],
        declarations: [
            NtNavSidebarComponent,
            NtNavTopbarComponent,
        ],
        providers: [NtNavigationAdapter]
    })
], NtNavModule);

export { NT_NAV_CONFIG, NtNavModule, NtNavSidebarComponent, NtNavTopbarComponent, NtNavUrlTreeStrategy, NtNavigationAdapter, generateNavMenus, resolveTitle };
//# sourceMappingURL=navigation.js.map
