import { InjectionToken, Inject, Injectable, ChangeDetectorRef, Input, Component, ViewEncapsulation, ChangeDetectionStrategy, NgModule } from '@angular/core';
import { OverlayModule } from '@angular/cdk/overlay';
import { CommonModule } from '@angular/common';
import { NavigationEnd, ActivatedRoute, Router, RouterModule } from '@angular/router';
import { Subject, BehaviorSubject } from 'rxjs';
import { takeUntil, filter } from 'rxjs/operators';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { NtOverlayModule } from '@ng-tangram/components/overlay';

var NtNavUrlTreeStrategy;
(function (NtNavUrlTreeStrategy) {
    NtNavUrlTreeStrategy[NtNavUrlTreeStrategy["Integrity"] = 0] = "Integrity";
    NtNavUrlTreeStrategy[NtNavUrlTreeStrategy["OnlyParents"] = 1] = "OnlyParents";
})(NtNavUrlTreeStrategy || (NtNavUrlTreeStrategy = {}));
var NT_NAV_CONFIG = new InjectionToken('nt-nav-config');

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

function getCjsExportFromNamespace (n) {
	return n && n['default'] || n;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

function generateNavMenus(routes, rootDir) {
    return routes.filter(function (route) { return !route.redirectTo; }).map(function (route) {
        var menuRoles = (route.canActivate && route.canActivate.map(function (activate) { return activate.ROLE_KEY; })) || [];
        var menu = { path: route.path, data: route.data, roles: menuRoles.flat(Infinity) || [] };
        var subModule = commonjsRequire("src/" + rootDir + "/" + route.path + "/" + route.path + ".module.ts");
        if (subModule.routes) {
            menu.children = subModule.routes.filter(function (route) { return !route.redirectTo; }).map(function (route) {
                var roles = (route.canActivate && route.canActivate.map(function (activate) { return activate.ROLE_KEY; })) || [];
                return { path: route.path, data: route.data, roles: roles.flat(Infinity) || [] };
            });
        }
        return menu;
    });
}
var RESOLVE_TITLE_PLACEHOLDER = /^resolve::/;
function resolveTitle(snapshot, title) {
    var _a;
    if (RESOLVE_TITLE_PLACEHOLDER.test(title)) {
        var paths = title.replace(RESOLVE_TITLE_PLACEHOLDER, '').split('.');
        var key = paths.shift();
        try {
            if (key && ((_a = snapshot.data) === null || _a === void 0 ? void 0 : _a.hasOwnProperty(key))) {
                var data = snapshot.data[key];
                var path = paths.shift();
                var result = path ? data[path] : null;
                while (paths.length > 0 && result instanceof Object) {
                    path = paths.shift();
                    result = path ? result[path] : null;
                }
                return result;
            }
            return key || '';
        }
        catch (_b) { }
    }
    return title;
}

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var NtNavigationAdapter = (function () {
    function NtNavigationAdapter(route, router, config) {
        var _this = this;
        this.route = route;
        this.router = router;
        this.config = config;
        this.destroy = new Subject();
        this.activatedUrlTree = new BehaviorSubject([]);
        this.activeUrl = new BehaviorSubject({});
        var routes = config.routes, rootDir = config.rootDir;
        this.menus = generateNavMenus(routes, rootDir);
        router.events
            .pipe(takeUntil(this.destroy), filter(function (event) { return event instanceof NavigationEnd; }))
            .subscribe(function () { return _this.buildUrlTree(_this.route); });
    }
    NtNavigationAdapter.prototype.ngOnDestroy = function () {
        this.destroy.next();
        this.destroy.complete();
    };
    NtNavigationAdapter.prototype.getRouteMenus = function () {
        return this.menus;
    };
    NtNavigationAdapter.prototype.buildUrlTree = function (route) {
        var url = [];
        var tree = [];
        while (route.firstChild) {
            route = route.firstChild;
            var snapshot = route.snapshot;
            if (snapshot.data && snapshot.data.title && snapshot.url.length > 0) {
                var data = snapshot.data, title = void 0;
                title = resolveTitle(snapshot, data.title);
                url = url.concat(snapshot.url.map(function (segment) { return segment.path; }));
                tree.push({ title: title, url: url, icon: data.icon });
            }
        }
        this.activeUrl.next(tree[tree.length - 1]);
        if (this.config.urlTreeStrategy === NtNavUrlTreeStrategy.OnlyParents) {
            tree.splice(tree.length - 1, 1);
        }
        this.activatedUrlTree.next(tree);
    };
    NtNavigationAdapter.ctorParameters = function () { return [
        { type: ActivatedRoute },
        { type: Router },
        { type: undefined, decorators: [{ type: Inject, args: [NT_NAV_CONFIG,] }] }
    ]; };
    NtNavigationAdapter = __decorate([
        Injectable(),
        __param(2, Inject(NT_NAV_CONFIG))
    ], NtNavigationAdapter);
    return NtNavigationAdapter;
}());

var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __decorate$1 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtNavSidebarComponent = (function () {
    function NtNavSidebarComponent(changeDetectorRef, navAdapter) {
        var _this = this;
        this.changeDetectorRef = changeDetectorRef;
        this.navAdapter = navAdapter;
        this.destroy = new Subject();
        this._dropdown = false;
        this._baseUrl = navAdapter.config.baseUrl || './';
        this._routeMenus = this._filterRouteMenus();
        this.navAdapter.activatedUrlTree
            .pipe(takeUntil(this.destroy), filter(function (tree) { return tree.length > 0; }))
            .subscribe(function (tree) { return _this._foldedActivatedMenu(tree); });
    }
    Object.defineProperty(NtNavSidebarComponent.prototype, "dropdown", {
        get: function () { return this._dropdown; },
        set: function (value) {
            this._dropdown = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtNavSidebarComponent.prototype, "filter", {
        get: function () {
            return this._filter;
        },
        set: function (value) {
            if (value) {
                this._filter = value;
                this._routeMenus = this._filterRouteMenus();
                this.changeDetectorRef.markForCheck();
            }
        },
        enumerable: true,
        configurable: true
    });
    NtNavSidebarComponent.prototype._foldedActivatedMenu = function (tree) {
        var moduleName = tree[0].url.join('/');
        var activated = this._routeMenus.find(function (menu) { return menu.path === moduleName; });
        activated && (activated.folded = true);
        this.changeDetectorRef.markForCheck();
    };
    NtNavSidebarComponent.prototype._filterRouteMenus = function () {
        var _this = this;
        if (typeof this.filter !== 'function') {
            return this.navAdapter.getRouteMenus();
        }
        var parents = this.navAdapter.getRouteMenus().filter(function (parent) {
            if (parent.children && parent.children.length > 0) {
                return true;
            }
            else {
                return _this.filter(parent);
            }
        });
        var value = parents.map(function (parent) {
            var copy = __assign({}, parent);
            if (copy.children) {
                copy.children = copy.children.filter(function (child) { return _this.filter(child); });
            }
            return copy;
        });
        var result = [];
        value.forEach(function (menuItem) {
            if (!menuItem.children || (menuItem.children && menuItem.children.length > 0)) {
                result.push(__assign({}, menuItem));
            }
        });
        return result;
    };
    NtNavSidebarComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef },
        { type: NtNavigationAdapter }
    ]; };
    __decorate$1([
        Input()
    ], NtNavSidebarComponent.prototype, "dropdown", null);
    __decorate$1([
        Input()
    ], NtNavSidebarComponent.prototype, "filter", null);
    NtNavSidebarComponent = __decorate$1([
        Component({
            selector: 'nt-nav-sidebar',
            template: "<ul class=\"nt-nav-sidebar-root-menu\">\n  <li *ngFor=\"let parent of _routeMenus\"\n    routerLinkActive=\"active\"\n    [class.folded]=\"parent.folded\"\n    [class.has-children]=\"parent.children\">\n\n    <ng-container *ngIf=\"!dropdown else dropdownMode\">\n      <a *ngIf=\"!parent.children else hasChildrenMenus\" [routerLink]=\"[_baseUrl, parent.path]\">\n        <i [class]=\"parent.data['icon']\"></i>\n        <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n      </a>\n      <ng-template #hasChildrenMenus>\n        <a (click)=\"parent.folded = !parent.folded\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n        </a>\n        <ul class=\"nt-nav-sidebar-nested-menu\">\n          <li *ngFor=\"let child of parent.children\" routerLinkActive=\"active\">\n            <a [routerLink]=\"[_baseUrl, parent.path, child.path]\">\n              <span class=\"nt-nav-sidebar-title\">{{ child.data['title'] }}</span>\n            </a>\n          </li>\n        </ul>\n      </ng-template>\n    </ng-container>\n\n    <ng-template #dropdownMode>\n      <ng-container *ngIf=\"!parent.children else hasChildrenMenus\">\n        <a #origin=\"cdkOverlayOrigin\"\n        cdkOverlayOrigin\n        (mouseenter)=\"overlay.markOpen()\"\n        (mouseleave)=\"overlay.markClose()\"\n        [routerLink]=\"[_baseUrl, parent.path]\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n\n          <nt-overlay\n            #overlay\n            overlayClass=\"nt-nav-dropdown-overlay\"\n            [origin]=\"origin\"\n            position=\"rightTop\"\n            [arrow]=\"true\"\n            (overlayEnter)=\"overlay.markOpen()\"\n            (overlayLeave)=\"overlay.markClose()\">\n            <div class=\"nt-nav-dropdown-pane\">\n              <div class=\"nt-nav-dropdown-title\">{{ parent.data['title'] }}</div>\n            </div>\n          </nt-overlay>\n        </a>\n      </ng-container>\n      <ng-template #hasChildrenMenus>\n        <a #origin=\"cdkOverlayOrigin\"\n          cdkOverlayOrigin\n          (mouseenter)=\"overlay.markOpen()\"\n          (mouseleave)=\"overlay.markClose()\">\n          <i [class]=\"parent.data['icon']\"></i>\n          <span class=\"nt-nav-sidebar-title\">{{ parent.data['title'] }}</span>\n          <nt-overlay\n            #overlay\n            overlayClass=\"nt-nav-dropdown-overlay\"\n            [origin]=\"origin\"\n            position=\"rightTop\"\n            [arrow]=\"true\"\n            (overlayEnter)=\"overlay.markOpen()\"\n            (overlayLeave)=\"overlay.markClose()\">\n            <div class=\"nt-nav-dropdown-pane\">\n              <div class=\"nt-nav-dropdown-title\">{{ parent.data['title'] }}</div>\n              <ul class=\"nt-nav-dropdown-menu\">\n                <li *ngFor=\"let child of parent.children\" routerLinkActive=\"active\">\n                  <a [routerLink]=\"[_baseUrl, parent.path, child.path]\">\n                    <span>{{ child.data['title'] }}</span>\n                  </a>\n                </li>\n              </ul>\n            </div>\n          </nt-overlay>\n        </a>\n      </ng-template>\n    </ng-template>\n  </li>\n</ul>\n\n",
            encapsulation: ViewEncapsulation.None,
            changeDetection: ChangeDetectionStrategy.OnPush,
            host: {
                'class': 'nt-nav-sidebar',
                '[class.dropdown-mode]': 'dropdown'
            }
        })
    ], NtNavSidebarComponent);
    return NtNavSidebarComponent;
}());

var __decorate$2 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtNavTopbarComponent = (function () {
    function NtNavTopbarComponent(navAdapter) {
    }
    NtNavTopbarComponent.ctorParameters = function () { return [
        { type: NtNavigationAdapter }
    ]; };
    NtNavTopbarComponent = __decorate$2([
        Component({
            selector: 'nt-nav-topbar',
            template: "",
            encapsulation: ViewEncapsulation.None,
            host: {
                'class': 'nt-nav-topbar'
            }
        })
    ], NtNavTopbarComponent);
    return NtNavTopbarComponent;
}());

var __decorate$3 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtNavModule = (function () {
    function NtNavModule() {
    }
    NtNavModule_1 = NtNavModule;
    NtNavModule.forRoot = function (config) {
        return {
            ngModule: NtNavModule_1,
            providers: [
                { provide: NT_NAV_CONFIG, useValue: config }
            ]
        };
    };
    var NtNavModule_1;
    NtNavModule = NtNavModule_1 = __decorate$3([
        NgModule({
            imports: [
                CommonModule,
                RouterModule,
                OverlayModule,
                NtOverlayModule,
            ],
            exports: [
                NtNavSidebarComponent,
                NtNavTopbarComponent,
            ],
            declarations: [
                NtNavSidebarComponent,
                NtNavTopbarComponent,
            ],
            providers: [NtNavigationAdapter]
        })
    ], NtNavModule);
    return NtNavModule;
}());

export { NT_NAV_CONFIG, NtNavModule, NtNavSidebarComponent, NtNavTopbarComponent, NtNavUrlTreeStrategy, NtNavigationAdapter, generateNavMenus, resolveTitle };
//# sourceMappingURL=navigation.js.map
