"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const tasks_1 = require("@angular-devkit/schematics/tasks");
const ast_1 = require("../utils/ast");
const angular_config_1 = require("../utils/angular-config");
const package_config_1 = require("../utils/package-config");
function setupForceOptions(projectName) {
    return (tree) => {
        const config = angular_config_1.readConfig(tree);
        const appConfig = angular_config_1.getAngularAppConfig(config, projectName);
        const appSchematicsConfig = appConfig.schematics['@schematics/angular:component'];
        appConfig.schematics = Object.assign(Object.assign({}, appConfig.schematics), { '@schematics/angular:component': Object.assign(Object.assign({}, appSchematicsConfig), { style: 'scss' }) });
        angular_config_1.writeConfig(tree, config);
    };
}
function addTangramToPackageJson(modulePath) {
    return (tree) => {
        package_config_1.addPackageToDependenciesNode(tree, '@angular/cdk', 'latest');
        package_config_1.addPackageToDependenciesNode(tree, '@ng-tangram/components', 'latest');
        package_config_1.addPackageToDependenciesNode(tree, '@ng-tangram/pro', 'latest');
        ast_1.addModuleImportToModule(tree, modulePath, 'NtNoopUploadModule', '@ng-tangram/components/noop-upload');
        ast_1.addModuleImportToModule(tree, modulePath, 'NtFormsModule.forRoot()', '@ng-tangram/components/forms');
        return tree;
    };
}
/** 增加 ngxs  */
function addNgxsToPackageJson(modulePath) {
    return (tree) => {
        // Add Dependencies
        [
            '@ngxs/form-plugin',
            '@ngxs/logger-plugin',
            '@ngxs/router-plugin',
            '@ngxs/storage-plugin',
            '@ngxs/store'
        ].forEach(pkg => {
            package_config_1.addPackageToDependenciesNode(tree, pkg, "latest");
        });
        // Add Development Dependencies
        package_config_1.addPackageToDevDependenciesNode(tree, "@ngxs/devtools-plugin", "latest");
        ast_1.addModuleImportToModule(tree, modulePath, 'NgxsModule.forRoot([], { developmentMode: true }),', '@ngxs/store');
        ast_1.addModuleImportToModule(tree, modulePath, 'NgxsReduxDevtoolsPluginModule.forRoot()', '@ngxs/devtools-plugin');
        return tree;
    };
}
function addFontAwesomeToPackageJson(projectSourceRoot) {
    return (tree) => {
        [
            "@fortawesome/angular-fontawesome",
            "@fortawesome/fontawesome-free",
            "@fortawesome/free-brands-svg-icons",
            "@fortawesome/free-regular-svg-icons",
            "@fortawesome/free-solid-svg-icons"
        ].forEach(pkg => {
            package_config_1.addPackageToDependenciesNode(tree, pkg, "latest");
        });
        [
            "./node_modules/@fortawesome/fontawesome-pro/css/fontawesome.min.css",
            "./node_modules/@fortawesome/fontawesome-pro/css/solid.min.css",
            "./node_modules/@fortawesome/fontawesome-pro/css/duotone.min.css",
        ].forEach(style => {
            angular_config_1.addStyle(tree, projectSourceRoot, style);
        });
        return tree;
    };
}
function addUniversalPackageToPackageJson() {
    return (tree) => {
        package_config_1.addPackageToDependenciesNode(tree, '@nguniversal/common', 'latest');
        package_config_1.addPackageToDependenciesNode(tree, '@nguniversal/express-engine', 'latest');
        package_config_1.addPackageToDependenciesNode(tree, '@gorniv/ngx-universal', 'latest');
        return tree;
    };
}
function addFoundationPackageToPackageJson() {
    return (tree) => {
        package_config_1.addPackageToDependenciesNode(tree, 'foundation-sites', 'latest');
        return tree;
    };
}
function default_1(options) {
    return (tree, context) => {
        const workspace = angular_config_1.getWorkspace(tree);
        if (!options.name) {
            options.name = angular_config_1.getDefaultAngularAppName(workspace);
        }
        const project = workspace.projects[options.name];
        if (project.projectType !== 'application') {
            throw new schematics_1.SchematicsException(`Add requires a project type of "application".`);
        }
        const sourcePath = core_1.join(project.sourceRoot);
        const rootTemplateSource = schematics_1.apply(schematics_1.url(`./files/${options.template}`), [
            schematics_1.applyTemplates({
                classify: core_1.strings.classify,
                dasherize: core_1.strings.dasherize,
                name: options.name
            }),
            schematics_1.move(sourcePath)
        ]);
        const modulePath = ast_1.findModuleFromOptions(tree, options);
        const mergedRules = [
            setupForceOptions(sourcePath),
            addFoundationPackageToPackageJson(),
            addTangramToPackageJson(modulePath),
            addNgxsToPackageJson(modulePath),
            addFontAwesomeToPackageJson(sourcePath),
            schematics_1.mergeWith(rootTemplateSource),
        ];
        if (options.ssr) {
            mergedRules.push(addUniversalPackageToPackageJson());
        }
        if (!options.skipInstall) {
            context.addTask(new tasks_1.NodePackageInstallTask());
        }
        return schematics_1.chain(mergedRules);
    };
}
exports.default = default_1;
//# sourceMappingURL=index.js.map